# ./addons/eventscripts/bavm/bavm.py

import random

import es
import gamethread
from cfglib import AddonCFG
from playerlib import getPlayer

class AvM(object):
    class Player(object):
        def __init__(self, userid):
            self.userid  = userid
            self.speed   = False
            self.flicker = False
            self.player  = getPlayer(userid)

        def fire(self, command):
            es.server.queuecmd('es_xfire %s %s' % (self.userid, command))

        def give(self, command):
            es.server.queuecmd('es_xgive %s %s' % (self.userid, command))

        def strip(self, team):
            self.fire('player_weaponstrip Kill')
            self.give('player_weaponstrip')
            self.fire('player_weaponstrip Strip')
            gamethread.queue(self._giveSpecial, team)

        def setGravity(self, value):
            self.fire('!self addoutput "gravity %s"' % value)

        def flickerLoop(self):
            self.cancelFlickerLoop()
            if self.flicker:
                self.flicker = False
                self.trySetColors(0, 0, 0, 10)
            else:
                self.flicker = True
                self.trySetColors(255, 255, 255, 100)
            gamethread.delayedname(1, 'assassins_flicker_%s' % self.userid, self.flickerLoop)

        def cancelFlickerLoop(self, unload=False):
            gamethread.cancelDelayed('assassins_flicker_%s' % self.userid)
            if unload:
                self.trySetColors(255, 255, 255, 255)

        def playerSpawn(self, team, winner):
            gravity = 1
            self.strip(team)
            self.disableBuyZone()
            if team == 2:
                speed   = 1.5
                gravity = 0.45
                if winner == 2:
                    speed   = 2.8
                    gravity = 0.25
                    self.player.setArmor(200)
                self.player.setHealth(150)
                self.player.setSpeed(speed)
                self.setGravity(gravity)
            else:
                self.player.setHealth(random.randint(100, 250))
                self.player.setModel('models/player/slow/tau_commander/slow_tau_commander.mdl')
                if winner == 3:
                    gravity = 0.45
                    self.player.setSpeed(1.5)
                    gamethread.queue(self._giveNades)
            self.setGravity(gravity)

        def playerHurt(self, flicker, attackTime, winner):
            if not self.speed:
                if flicker:
                    self.cancelFlickerLoop()
                self.setGravity(1)
                self.player.setColor(255, 255, 255, 255)
                self.speed = True
                self.player.setSpeed(1)
                gamethread.delayed(attackTime, self._setNormal, (flicker, winner))

        def trySetColors(self, r, g, b, a):
            self.player.setColor(r, b, g, a)
            try:
                self.player.SetWeaponColor(r, b, g, a)
            except:
                pass

        def enableBuyZone(self):
            self.fire('func_buyzone Enable')

        def disableBuyZone(self):
            self.fire('func_buyzone Disable')

        def _setNormal(self, flicker, winner):
            speed      = 2
            gravity    = 0.45
            self.speed = False
            if winner == 2:
                speed = 2.8
                gravity = 0.25
                self.player.setArmor(200)
            self.setGravity(gravity)
            self.player.setSpeed(speed)
            if flicker:
                self.flickerLoop()

        def _giveNades(self):
            self.give('weapon_hegrenade')
            self.give('weapon_flashbang')
            self.give('weapon_smokegrenade')

        def _giveWeapons(self):
            self.give('weapon_%s' % random.choice(('glock', 'usp', 'fiveseven', 'p228', 'deagle', 'elite')))
            self.give('weapon_%s' % random.choice(('m3', 'xm1014', 'tmp', 'mac10', 'mp5navy', 'ump45', 'p90', 'famas', 'galil', 'ak47', 'm4a1', 'sg552', 'aug', 'scout', 'g3sg1', 'sg550', 'awp', 'm249')))

        def _giveSpecial(self, team):
            self.give('weapon_knife')
            self.give('item_assaultsuit')
            if team == 3:
                gamethread.queue(self._giveWeapons)

    def __init__(self):
        self.config     = None
        self.attackTime = None
        self.flicker    = None
        self.winner     = 0
        self.roundCount = 0
        self.prefix     = '#green[#lightgreenAssassins#green]'
        self.players    = {}

    def scriptLoad(self):
        self.config = AddonCFG(es.getAddonPath('bavm') + '/bavm_config.cfg')
        self.config.text('*********************************************************************************************')
        self.config.text('* Assassins vs. Machines Remake by BackRaw (original script by -=BoDzSaR=-) - Configuration *')
        self.config.text('*********************************************************************************************')
        self.config.text('')
        self.attackTime = self.config.cvar('avm_attacktime', 3, 'The time that an assassn will be back to their speed and jump when they hit somebody')
        self.flicker    = self.config.cvar('avm_flicker', 1, 'Should the Assassins always flicker in visibility? 1/0')
        self.config.text('\n')
        self.config.text('*********************************************************************************************')
        self.config.text('* Assassins vs. Machines Remake by BackRaw (original script by -=BoDzSaR=-) - Configuration *')
        self.config.text('*********************************************************************************************')
        self.config.write()
        self.executeConfig()
        self.doDownloads()
        for userid in es.getUseridList():
            self.insertUserid(userid)
        es.ServerVar('assassins_ver', 2.0, "BackRaw's Assassins vs. Machines Remake (original by -=BoDzSaR=-").makepublic()

    def scriptUnload(self):
        self.removeUserid(unload=True)

    def insertUserid(self, userid):
        userid = str(userid)
        self.players[userid] = self.Player(userid)

    def removeUserid(self, userid='0', unload=False):
        if not unload:
            self.players[userid].cancelFlickerLoop()
            del self.players[userid]
        else:
            if int(self.flicker):
                for userid in self.players:
                    x = self.players[userid]
                    x.cancelFlickerLoop(True)
                    x.enableBuyZone()
            self.players.clear()

    def executeConfig(self):
        self.config.execute()

    def doDownloads(self):
        es.stringtable('downloadables', 'materials/models/player/slow/tau_commander/slow_body.vmt')
        es.stringtable('downloadables', 'materials/models/player/slow/tau_commander/slow_body.vtf')
        es.stringtable('downloadables', 'materials/models/player/slow/tau_commander/slow_body_bump.vtf')
        es.stringtable('downloadables', 'materials/models/player/slow/tau_commander/slow_head.vmt')
        es.stringtable('downloadables', 'materials/models/player/slow/tau_commander/slow_head.vtf')
        es.stringtable('downloadables', 'materials/models/player/slow/tau_commander/slow_head_bump.vtf')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.dx80.vtx')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.dx90.vtx')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.mdl')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.phy')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.sw.vtx')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.vvd')
        es.stringtable('downloadables', 'models/player/slow/tau_commander/slow_tau_commander.xbox.vtx')

    def giveNade(self, userid, name):
        self.players[userid].give('weapon_%s' % name)

    def stripPlayer(self, userid, team):
        if userid in self.players:
            self.players[userid].strip(team)

    def playerSpawn(self, userid, team):
        if team > 1 and userid in self.players:
            self.players[userid].playerSpawn(team, self.winner)

    def playerHurt(self, attacker):
        self.players[attacker].playerHurt(int(self.flicker), int(self.attackTime), self.winner)

    def roundStart(self):
        for userid in self.players:
            self.players[userid].disableBuyZone()

    def roundFreezeEnd(self):
        for userid in filter(lambda x: es.getplayerteam(x) == 2, self.players):
            x = self.players[userid]
            x.trySetColors(0, 0, 0, 10)
            if int(self.flicker):
                x.flickerLoop()

    def roundEnd(self, winner):
        if winner in (2, 3):
            self.winner = winner
            self.roundCount += 1
            if self.roundCount == 20:
                self.roundCount = 0
                gamethread.delayed(.5, self._chooseTeam)
            if winner == 3:
                es.msg('#multi', self.prefix, 'Machines #defaulthave won and have stolen some assassin powers.')
            else:
                es.msg('#multi', self.prefix, '#defaultCongrulations your mission was a success. Please use these advanced skills wiseley.')
        else:
            self.winner = 0

    def _chooseTeam(self):
        for userid in self.players:
            es.changeteam(userid, 2 if es.getplayerteam(userid) == 3 else 3)
avm = AvM()

def server_cvar(ev):
    if ev['cvarname'].startswith('avm_'):
        avm.executeConfig()

def es_map_start(ev):
    avm.doDownloads()

def round_start(ev):
    avm.roundStart()

def round_freeze_end(ev):
    avm.roundFreezeEnd()

def round_end(ev):
    avm.roundEnd(int(ev['winner']))

def player_activate(ev):
    avm.insertUserid(ev['userid'])

def player_spawn(ev):
    avm.playerSpawn(ev['userid'], int(ev['es_userteam']))

def player_hurt(ev):
    if ev['es_attackerteam'] == '2':
        avm.playerHurt(ev['attacker'])

def player_disconnect(ev):
    avm.removeUserid(ev['userid'])

def hegrenade_detonate(ev):
    avm.giveNade(ev['userid'], 'hegrenade')

def flashbang_detonate(ev):
    avm.giveNade(ev['userid'], 'flashbang')

def smokegrenade_detonate(ev):
    avm.giveNade(ev['userid'], 'smokegrenade')

def item_pickup(ev):
    team = int(ev['es_userteam'])
    if team == 2 and ev['item'] != 'knife':
        avm.stripPlayer(ev['userid'], team)

def load():
    avm.scriptLoad()

def unload():
    avm.scriptUnload()
